<?php

/**
 * @file
 * Contains \Drupal\hamlets\Installer\CesConfigureForm.
 */

namespace Drupal\ces\Installer;

use Drupal\Core\Extension\ExtensionDiscovery;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\field\Entity\FieldConfig;
use Drupal\address\Entity\AddressFormat;
use Symfony\Component\DependencyInjection\ContainerInterface;

ini_set('max_execution_time', 150);

/**
 * Provides various localisation features.
 */
class CesConfigureForm extends ConfigFormBase {

  private $installer;
  private $moduleHandler;
  private $countryManager;
  private $addressCountries;
  private $routeBuilder;

  /**
   * Constructs a new SiteConfigureForm.
   *
   * @param Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param Drupal\Core\Extension\ModuleInstallerInterface $module_installer
   * @param Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   * @param Drupal\Core\Locale\CountryManagerInterface $country_manager
   * @param Drupal\address\CountryRepository $addressModule_countries
   * @param Drupal\Core\Routing\RouteBuilder
   *
   */
  public function __construct($config_factory, $module_installer, $module_handler, $country_manager, $addressModule_countries, $route_builder) {
    $this->setConfigFactory($config_factory);
    $this->installer = $module_installer;
    $this->moduleHandler = $module_handler;
    $this->countryManager = $country_manager;
    $this->addressCountries = $addressModule_countries;
    $this->routeBuilder = $route_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('module_installer'),
      $container->get('module_handler'),
      $container->get('country_manager'),
      $container->get('address.country_repository'),
      $container->get('router.builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId()  {
    return 'ces_configure_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#title'] = $this->t('Hamlets setup');


    //find all the cforge features
    $discovery = new ExtensionDiscovery(\Drupal::root());
    $available = $discovery->scan('module');
    foreach (system_rebuild_module_data() as $module_name => $extension) {
      $info = $extension->info;
      if ($info['package'] == $this->t('Cforge features')) {
        $feature_names[$module_name] = $info['name'];
        $feature_descriptions[$module_name] = $info['description'];
      }
    }
    $form['features'] = [
      '#title' => (string)$this->t('Hamlets features'),//hope that yields the current language
      '#type' => 'checkboxes',
      '#options' => $feature_names,
      '#default_value' => array_keys($feature_names),//everything on
      '#states' => [
        'disabled' => [
          ':input[name="populate"]' => ['value' => 'example']
        ]
      ],
      '#weight' => 1,
    ];
    
    $form['populate'] = [
      '#title' =>  $this->t('Import from CES Australia'),
      '#description' =>  $this->t('Enables all features first. Requires devel module'),
      '#type' => 'details',
      'exchange_id' => [
        '#title' => $this->t('CES exchange ID'),
        '#description' => $this->t('Four letters'),
        '#type' => 'textfield',
        '#max_length' => 4,
        '#width' => 4
      ],
      'pass' => [
        '#title' => $this->t('Password of 0000 account'),
        '#type' => 'textfield',
        '#weight' => 1
      ]
    ];

    $country = strtoupper($this->config('system.date')->get('country.default'));
    $options = \Drupal::service('address.country_repository')->getList();

    if (isset($options[$country])) {
      $form['country'] = [
        '#type' => 'value',
        '#value' => $country
      ];
    }
    else {
      $form['country'] = [
        '#type' => 'select',
        '#title' => $this->t('Country'),
        '#options' => $options,
        '#required' => TRUE,
        '#size' => 12,
      ];
    }

    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Save and continue'),
      '#weight' => 15,
      '#button_type' => 'primary',
    );

    \Drupal\cforge\Form\Settings::buildLoginOptions($form);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $modules[] = $form_state->getValue('flavour');

    if ($form_state->getValue('populate') == 'example') {
      $modules = array_keys($form['features']['#options']);
      $modules[] = 'dblog';
      $modules[] = 'views_ui';
      $modules[] = 'field_ui';
      $modules[] = 'admin_toolbar_tools';
      $modules[] = 'devel';
      $modules[] = 'devel_generate';

      \Drupal::configFactory()
        ->getEditable('views.settings')
        ->set('ui.show.sql_query.enabled', 1)
        ->set('ui.always_live_preview', 1)
        ->set('ui.show.advanced_column', 1)
        ->set('ui.show.master_display', 1)
        ->save();
    }
    else {
      $modules = array_keys(array_filter($form_state->getValue('features')));
      if (in_array('ces', $modules) and $form_state->getValue('populate') == 'ces_import') {
        $modules[] = 'ces_import';
      }
      else{
        $modules[] = 'mcapi_signatures';
      }
    }

    $this->installer->install($modules);
    drupal_set_message('installed modules '.implode(',', $modules));

    //for the ces module, remove the categories field from transactions
    if (\Drupal::moduleHandler()->moduleExists('ces')) {
      if ($field = \Drupal\field\Entity\FieldStorageConfig::load('mcapi_transaction.categories')) {
        $field->delete();
      }
    }

    $terms = \Drupal::entityTypeManager()
      ->getStorage('taxonomy_term')
      ->loadByProperties(
        ['vid' => SMALLAD_CATEGORIES]
      );
    //delete any existing terms in the vocab (one is installed by default)
    foreach ($terms as $term) {
      drupal_set_message('Deleted term: '.$term->label());
      $term->delete();
    }

    if (substr($form_state->getValue('populate'), 0, 6) != 'import') {
      if ($neighbourhoods = $form_state->getValue('neighbourhoods')) {
        foreach (array_unique(explode("\n", $neighbourhoods)) as $name) {
          if(\Drupal::entityTypeManager()
            ->getStorage('node')->getQuery()
            ->condition('type', 'neighbourhood')
            ->condition('title', $name)
            ->execute()) {
            continue;
          }
          \Drupal\node\Entity\Node::create([
            'type' => 'neighbourhood',
            'title' => trim($name)
          ])->save();
        }
      }

      if ($form_state->getValue('populate') == 'example' and !$form_state->getValue('hamlets_vocab')) {
         $form_state->setValue('hamlets_vocab', 'English LETS');
      }

      if ($preset = $form_state->getValue('hamlets_vocab'))  {
        //now import the terms into the vocab
        module_load_install('cforge');
        $vocab = Vocabulary::load(SMALLAD_CATEGORIES);
        $this->vocabImportRecurisive(
          $vocab,
          $this->hamletsVocabs()[$preset]
        );
        $vocab->save();//thie hierarchical property
      }


    }

    //realname module settings @todo this should be overridden with a config file
    \Drupal::configFactory()
      ->getEditable('realname.settings')
      ->set('pattern', '[user:firstname] [user:lastname]')
      ->set('login', TRUE)
      ->save();


    \Drupal\user\Entity\User::load(1)
      ->set('firstname', 'Hamlets')
      ->set('lastname', 'Admin')
      ->save();

    //finally set the addressfield to the country's addressFormat
    $this->countryCode = $form_state->getValue('country');
    $field = FieldConfig::load('user.user.address');
    $settings = $field->get('settings');
    $settings['available_countries'][$this->countryCode] = $this->countryCode;
    $field->set('settings', $settings)->save();
    Self::modifyAddressFormat($this->countryCode);


    if ($form_state->getValue('populate') == 'example') {
      $this->develGenerate();
      //@todo this should be moved to the end of the batch, when this job is batched.
      \Drupal::service('module_installer')->uninstall(['devel_generate']);
    }

  }


  /**
   * @see https://www.drupal.org/node/2503429 for possible reason why batching doesn't work
   * In the mean time we have to stay within the batch limits of each plugin
   * users:no limit
   * content: 50
   * transactions:100
   * smallads: 60
   */
  function develGenerate() {
    /**
    $batch = ['title' => 'Generating example content'];
    //create 26 users
    $batch['operations'][] = [
      'cforge_install_run_batch',
      [
        'user',
        ['kill' => TRUE, 'num' => 26, 'pass' => 'password', 'roles' => ['trader'], 'time_range' => strtotime('-2 years')]
      ]
    ];
    $batch['operations'][] = [
      'cforge_install_run_batch',
      [
        'smallad',
        ['kill' => TRUE, 'num' => 120, 'type' => 'offer', 'since' => strtotime('-6 months')]
      ]
    ];
    $batch['operations'][] = [
      'cforge_install_run_batch',
      [
        'smallad',
        ['kill' => TRUE, 'num' => 40, 'type' => 'want', 'since' => strtotime('-6 months')]
      ]
    ];
    $batch['operations'][] = [
      'cforge_install_run_batch',
      [
        'mcapi_transaction',
        ['kill' => TRUE, 'num' => 200, 'type' => 'default']
      ]
    ];
    $batch['operations'][] = [
      'cforge_install_run_batch',
      [
        'content',
        ['kill' => FALSE, 'num' => 15, 'node_types' => ['story' => TRUE], 'max_comments' => 6, 'title_length' => 8]
      ]
    ];
    $batch['operations'][] = [
      'cforge_install_run_batch',
      [
        'content',
        ['kill' => FALSE, 'num' => 15, 'node_types' => ['event' => TRUE], 'max_comments' => 2, 'title_length' => 8]
      ]
    ];
    $batch['operations'][] = [
      'cforge_install_run_batch',
      [
        'content',
        ['kill' => FALSE, 'num' => 15, 'node_types' => ['document' => TRUE], 'max_comments' => 0, 'title_length' => 5]
      ]
    ];
    $batch['operations'][] = [
      'cforge_install_run_batch',
      [
        'content',
        ['kill' => FALSE, 'num' => 50, 'node_types' => ['image' => TRUE], 'max_comments' => 4, 'title_length' => 5]
      ]
    ];
    $batch['operations'][] = [
      'RandomNeighbourhoods', []
    ];
    batch_set($batch);
    return;
     *
     */


    $develGen = \Drupal::service('plugin.manager.develgenerate');

    //create 26 users
    //switch to an authenticated user id otherwise the generated wallet names will be wrong
    \Drupal::service('account_switcher')->switchTo(\Drupal\user\Entity\User::load(1));
    $develGen->createInstance('hamlets_user', [])
      ->generate(['kill' => TRUE, 'pass' => 'password', 'roles' => ['trader'], 'time_range' => strtotime('-2 years')]);
    \Drupal::service('account_switcher')->switchBack();

    $this->RandomNeighbourhoods();

    //create 35 wants
    $develGen->createInstance('smallad', [])
      ->generate(['kill' => FALSE, 'num' => 35, 'type' => 'want', 'since' => strtotime('-6 months')]);

    //create 120 offers
    $develGen->createInstance('smallad', [])
      ->generate(['kill' => FALSE, 'num' => 60, 'type' => 'offer', 'since' => strtotime('-6 months')]);
    $develGen->createInstance('smallad', [])
      ->generate(['kill' => FALSE, 'num' => 60, 'type' => 'offer', 'since' => strtotime('-6 months')]);

    //create 200 default transactions
    $develGen->createInstance('mcapi_transaction', [])
      ->generate(['kill' => TRUE, 'num' => 100, 'type' => 'default']);
    $develGen->createInstance('mcapi_transaction', [])
      ->generate(['kill' => FALSE, 'num' => 100, 'type' => 'default']);

    //create 20 stories
    $develGen->createInstance('content', [])
      ->generate(['kill' => FALSE, 'num' => 15, 'node_types' => ['story' => TRUE], 'max_comments' => 6, 'title_length' => 8]);
    //create 10 events
    //does this make events in the past or future?
    $develGen->createInstance('content', [])
      ->generate(['kill' => FALSE, 'num' => 15, 'node_types' => ['event' => TRUE], 'max_comments' => 2, 'title_length' => 8]);
    //because the date field generates dates within the last year, add 6 months to every node
    $events = \Drupal::entityTypeManager()->getStorage('node')->loadByProperties(['type'=> 'event']);
    $addition = 3600*24*183;
    foreach ($events as $event) {
      $time = REQUEST_TIME + mt_rand(-86400*180, 86400*180);
      $event->date->value = gmdate(DATETIME_DATETIME_STORAGE_FORMAT, $time);
      $event->save();
    }

    //create 15 documents
    $develGen->createInstance('content', [])
      ->generate(['kill' => FALSE, 'num' => 15, 'node_types' => ['document' => TRUE], 'max_comments' => 0, 'title_length' => 5]);

    //create another 3 galleries
    $props = ['vid' => 'galleries', 'format' => 'plain_text'];
    Term::create($props + [
      'name' => t('Annnual street party'),
    ])->save();
    Term::create($props + [
      'name' => t('Gurning competition'),
    ])->save();
    Term::create($props + [
      'name' => t('Harvest'),
    ])->save();

    //now create 50 photos
    $develGen->createInstance('content', [])
      ->generate(['kill' => FALSE, 'num' => 50, 'node_types' => ['image' => TRUE], 'max_comments' => 4, 'title_length' => 5]);

  }

  //not used
  function cforge_install_run_batch($batch, $plugin, $args = []) {
    $develGen = \Drupal::service('plugin.manager.develgenerate')
      ->createInstance($plugin)
      ->generate($args);
  }

  function RandomNeighbourhoods(&$batch = []) {
    //put all the users in random neighbourhoods
    $hood_nids = \Drupal::entityTypeManager()//@todo inject this
      ->getStorage('node')->getQuery()
      ->condition('type', 'neighbourhood')
      ->execute();

    foreach (\Drupal\node\Entity\Node::loadMultiple($hood_nids) as $hood) {
      $hood_names[] = $hood->label();
    }
    if ($hood_nids) {
      foreach (\Drupal\user\Entity\User::loadMultiple() as $uid => $account) {
        if (!$account->id()) {
          continue;
        }
        $address = $account->address->getValue();
        $address[0]['dependent_locality'] = $hood_names[array_rand($hood_names)];
        $address[0]['countryCode'] = $this->countryCode;
        $account->set('address', $address)->save();
      }
    }
    else drupal_set_message('No neighbourhoods found.', 'error');
  }

  static function modifyAddressFormat($countryCode) {
    $format = AddressFormat::load($countryCode);
    if (!$format) {
      echo 'failed to load country '.$countryCode;
    }
    //if we have an altered version of the address, overwrite that country's address with ours
    if ($alt = AddressFormat::load('CF-'.$countryCode)) {
      foreach ($alt->toArray() as $key => $value) {
        if ($key != 'countryCode' && $key != 'uuid') {
          $format->set($key, $value);
        }
      }
      $format->save();
    }
    //if we don't then bodge the existing format
    else {
      $pattern = $format->getFormat();
      $pattern .="\n%dependentLocality";
      $format->setFormat($pattern)
        ->setrequiredFields(['dependentLocality'])
        ->save();
      drupal_set_message('Resaved addressformat '. $format->id());
    }
    if (\Drupal::moduleHandler()->moduleExists('ces')) {
      module_load_install('ces');
      //ces_show_org_subfield($countryCode);
    }
  }


  private function vocabImportRecurisive($vocab, $termArray, $parent = 0) {
    $w = $hierarchy = 0;
    foreach (Element::Children($termArray) as $name) {
      $props = [
        'name' => $name,
        'weight' => $w++,
        'vid' => $vocab->id(),
        'parent' => $parent,
        'format' => 'plain_text'
      ];
      $term = Term::Create($props)->save();
      if ($termArray['#children']) {
        $vocab->setHierarchy(TAXONOMY_HIERARCHY_SINGLE)->value = TRUE;
        $this->vocabImportRecurisive($vocab, $termArray[$name], $term->tid);
      }
    }

  }

  public function hamletsVocabs() {
    $vocabs = [
      'English LETS' => [
        '#children' => FALSE,
        'Arts & Culture' => ['tango', 'show', 'storytelling', 'clown', 'improvis', 'cinema', 'expression', 'flyer', 'brochure', 'culptures', 'photographer', 'album', 'book', 'culture', 'novel', 'voice', 'reading', 'poetry', 'books', 'writing', 'drawing', 'photog'],
        'Business Services & Clerical' => ['writing', 'papers', 'clarif', 'admini', 'council', 'office', 'classif', 'correspond', 'photocop', 'mail', 'marketing', 'publicity', 'spelling', 'correction', 'syntax', 'translat', 'networking', 'content', 'reorgani', 'minutes', 'proofread', 'spelling'],
        'Clothing' => ['jeans', 'skirts', 'dress', 'sock', 'purse', 'apron', 'dress', 'watches', 'sandals', 'shirt', 'shoes', 'shirt', 'jacket', 'handbags', 'coat', 'jacket', 'waist', 'jacket', 'glove'],
        'Computing & Electronics' => ['comput', 'disc', 'disk', 'music', 'download', 'digital', 'office', 'management', 'FileMaker', 'given', 'digital', 'electronic', 'internet', 'email', 'camera', 'phone', 'software', 'Mac', 'audio', 'digitising', 'Excel', 'Quickbooks', 'Photoshop', 'mobile', 'MP3', 'movie', 'radio', 'wireless', 'wi-fi', 'phone', 'CD', 'DVD'],
        'Education & Language' => ['German', 'Italian', 'language', 'training', 'lessons', 'French', 'school', 'tutor'],
        'Food' => ['Bread', 'Dough', 'mixer', 'kitchen', 'stove', 'fungus', 'vinegar', 'egg', 'salad', 'organic', 'honey', 'rosemary', 'tart', 'sweet', 'biscuit', 'thyme', 'syrup', 'dessert', 'seed', 'pesto', 'vegetable', 'couscous', 'semolina', 'meal', 'glass', 'casserole', 'dish', 'soup', 'pastry', 'mint', 'sauce', 'caviar', 'elderberry', 'drink', 'grinder', 'jelly', 'jam', 'juice', 'drink', 'recipe', 'pie', 'veg'],
        'Health & Wellness' => ['beauty', 'childre', 'haircut', 'scrub', 'spirit', 'massage', 'therapy', 'shaman', 'healer', 'acne', 'Reiki', 'meditation', 'health', 'genealogical', 'disability', 'babysitting', 'aloe', 'fascia', 'acupuncture', 'therapy', 'osteopathy', 'homeopathic', 'listening', 'difficulties', 'muscle', 'dyeing', 'removal', 'migraine', 'sensory', 'stress', 'cabinet', 'gymnastic'],
        'House & garden' => ['carpet', 'balcony', 'decor', 'lawn', 'mower', 'ironing', 'Garland', 'Yucca', 'porcelain', 'fridge', 'stove', 'washer', 'mill', 'corkscrew', 'carpet', 'cleaning', 'papyrus', 'ceiling', 'chair', 'ground', 'sofa', 'pillow', 'library', 'chairs', 'cleaning', 'glass', 'vacuum', 'cleaner', 'freezer', 'shelves', 'interior', 'soil', 'water', 'closet', 'garden', 'household', 'cleaning'],
        'Sports & Leisure' => ['scuba', 'walking', 'rental', 'punch', 'rucksack', 'GPS', 'sailboat', 'overnight', 'hiking', 'snowshoeing', 'canoe', 'rental', 'walking', 'ski', 'snowboard', 'snowshoe', 'hiking', 'trip', 'discover', 'vacancy'],
        'Skills & DIY' => ['embroidery', 'saw', 'sanding', 'lacquer', 'varnish', 'electric', 'adjustment', 'arrangement', 'floor', 'decoration', 'makeover', 'necklace', 'jewelry', 'semi-precious', 'bracelet', 'loops', 'sculpture', 'watercolor', 'canvas', 'acrylic', 'floral', 'fantasy', 'craft', 'beads', 'repair', 'fabric', 'paint', 'tool', 'renovation', 'work', 'Carpenter', 'roof', 'heating'],
        'Transport' => ['bicycle', 'bike', 'helmet', 'transport', 'airport', 'bike', 'scooter', 'bicycle', 'van', 'minibus', 'transport', 'moving', 'trailer', 'ticket'],
        'Miscellaneous' => []
      ],
      'SEL Francais' => [
        '#children' => FALSE,
        'Administration' => ['restructuration', 'réorganisation', 'dactylographie', 'feuille', 'traduction', 'secrétariat', 'syntaxe', 'marketing', 'correction', 'orthographe', 'publicité', 'mail', 'photocop', 'correspondance', 'classement', 'bureau', 'conseil', 'administrative', 'clarification', 'Rédaction', 'paperasses', 'réseau', 'contenu', 'procès-verbaux', 'orthographe', 'correction', 'Relecture'],
        'Alimentation' => ['herbier', 'Pâte', 'mixer', 'Pain', 'Cuisine', 'champignon', 'cuisinière', 'vinaigre', 'Orgonite', 'aspergettes', 'salade', 'oeuf', 'miel', 'bio', 'betty bossy', 'ravioli', 'Tarte', 'Tapenade', 'romarin', 'Laurier', 'bonbon', 'biscuit', 'Desserts', 'graine', 'Thym', 'Sirop', 'pesto', 'légume', 'semoule', 'couscous', 'levain', 'épeautre', 'germoir', 'Repas', 'Tisane', 'verres', 'casserole', 'Soupe', 'Plat', 'Feuilleté', 'menthe', 'courgeépices', 'raves', 'compote', 'râppe', 'boisson', 'Caviar', 'sureau', 'Hachoir', 'confiture', 'vera', 'Gelée', 'boisson', 'Jus', 'recette', 'pâté', 'végét', 'gomasio', 'Liqu'],
        'Artisanat & Bricolage' => ['scie', 'ponçeuse', 'broderies', 'ponçage', 'Réglage', 'laque', 'vernis', 'électrique', 'parquet', 'arrangement', 'décoration', 'collier', 'relooking', 'bijoux', 'pierre', 'semi-précieuse', 'bracelet', 'aquarelle', 'catels', 'boucles', 'sculpture', 'Toile', 'acrylique', 'Floraux', 'fantaisie', 'bricolage', 'réparation', 'perles', 'peinture', 'art', 'Tissu', 'perçuese', 'outil', 'petits travaux', 'Menuisier', 'etau', 'rénovation', 'montage', 'revêtement', 'toiture', 'fumisterie', 'chauffage'],
        'Arts & Culture' => ['spectacle', 'tango', 'clown', 'conte', 'mprovisation', 'cinéma', 'expression', 'dépliant', 'flyer', 'brochure', 'culpture', 'photographe', 'album', 'alligraphie', 'fusain', 'livre', 'Couture', 'roman', 'livres', 'poesie', 'voix', 'Lecture', 'dessin', 'ecriture', 'invention', 'photographie'],
        'Cours & Langues' => ['allemand', 'anglais', 'devoir', 'langue', 'italien', 'formation', 'eçons', 'cours', 'Répétiteur', 'Soutien scolaire', 'français', 'animation', 'nitiation'],
        'Informatique & Eléctro' => ['nformatique', 'télécharger', 'disco', 'compile', 'musique', 'bureautique', 'numérique', 'gestion', 'web', 'digital', 'FileMaker', 'donnée', 'nternet', 'email', 'électronique', 'ordinateur', 'caméra', 'téléphone', 'Mac', 'logiciels', 'software', 'audio', 'numériser', 'diapositive', 'Excel', 'Word', 'Quickbooks', 'MP3', 'enregistrement', 'hotoshop', 'natel', 'lecteur', 'film', 'radio', 'haut-parleur', 'wireless', 'wi-fi', 'CD', 'DVD', 'phone'],
        'Maison & Jardin' => ['tapis', 'biblioth', 'balcon', 'sommier', 'décor', 'gazon', 'tondeuse', 'repasser', 'Guirlande', 'Yucca', 'porcelaine', 'Frigo', 'vaisselle', 'poêle', 'poivrier', 'moulin', 'hache', 'tire-bouchon', 'moquette', 'tapis', 'nettoyage', 'papyrus', 'plafond', 'mur', 'fauteuil', 'duvet', 'drap', 'canapé', 'terre', 'biblio', 'oreiller', 'vitre', 'aspirateur', 'Ménage', 'chaises', 'nettoyeur', 'plante', 'Étagères', 'matelas', 'parquet', 'congélateur', 'aménagement', 'intérieur', 'sol', 'boiserie', 'meuble', 'armoire', 'nappe', 'jardin', 'ménage', 'drap', 'nettoyage'],
        'Mobilité' => ['vélo', 'bike', 'transport', 'casque', 'CFF', 'Voiturage', 'aéroport', 'moto', 'scooter', 'bicycle', 'camion', 'van', 'Minibus', 'trajets', 'campagne', 'transport', 'voiture', 'velo', 'déménagement', 'remorque'],
        'Santé & Soins' => ['viscéral', 'beauté', 'soignant', 'Garde d\'enfants', 'haptonomie', 'Kinésio', 'cupress', 'coupe de cheveux', 'masque', 'sacrée', 'gommage', 'spirit', 'massage', 'traitement', 'coachin', 'guériss', 'shaman', 'acnée', 'Huile', 'Reiki', 'méditation', 'santé', 'généalog', 'handicap', 'baby-sitting', 'aloe', 'fascia', 'thérapie', 'Soins', 'Acupuncture', 'Ostéopathie', 'homéopathiques', 'écoute', 'Difficultés', 'musculaire', 'teinture', 'épilation', 'apprivoiser', 'migraine', 'sensoriel', 'stress', 'Cabinet', 'gymnasti'],
        'Sports & Evasion' => ['scuba', 'location', 'séjour', 'promenade', 'sac de frappe', 'loger', 'GPS', 'voilier', 'raquette', 'Canoë', 'nuitée', 'randonnée', 'location', ' tente', 'marche', 'Löwa', 'ski', 'snowboard', 'balade', 'Tenue', 'plongée', 'raquettes', 'randonnées', 'ballade', 'découvert', 'voyage', 'vacance', 'sortie'],
        'Vêtements plus' => ['vêtement', 'chaussette', 'Jeans', 'jupon', 'robe', 'sac à main', 'Tablier', 'sandales', 'montres', 'habits', 'chaussure', 'pantalon', 'shirt', 'chemise', 'Sacsà main', 'doudoune', 'blouse', 'veste', 'ceinture', 'manteau', 'gant', 'veston', 'gilet', 'chausettes', 'bonnet', 'chapeau'],
        'Divers' => [],
      ],
      'SEL Belgique, (2 Tier]' => [
        '#children' => TRUE,
        'A Ménage / Entretien maison' => [
          '#children' => FALSE,
          "Nettoyage/ produits d'entretien" => [],
          'Lessive, repassage' => [],
          'Couture, tricot' => [],
          'Rangement' => [],
          'Ménage - Divers' => []
        ],
        'B Travaux maisons' => [
          '#children' => FALSE,
          'Travaux lourds' => [],
          'Petits travaux divers' => [],
          'Peinture, tapissage, décoration' => [],
          'Electricité' => [],
          'Plomberie, chauffage' => [],
          'Carrelage, plafonnage, maçonnerie' => [],
          'Menuiserie, planchers, meubles' => [],
          "Isolation, économies d'énergie" => [],
          "Travaux - Divers" => [],
        ],
        'C Enfants / Ados' => [
          '#children' => FALSE,
          'Grossesse et bébé' => [],
          'Baby sitting' => [],
          'Stages, animations créatives' => [],
          'Aide scolaire' => [],
          'Jeux et jouets' => [],
          'Enfants - Divers' => [],
        ],
        'D Alimentation / Gastronomie' => [
          '#children' => FALSE,
          'Nettoyage fruits et légumes' => [],
          'Boissons, soupes, sauces' => [],
          'Plats' => [],
          'Desserts' => [],
          "Tables d'hôtes" => [],
          'Cours, conseils, recettes' => [],
          'Alimentation - Divers' => []
        ],
        'E Santé / Bien-être / Accompagnement' => [
          '#children' => FALSE,
          'Coiffure, esthétique visage et corps' => [],
          'Massage, thérapies, produits naturels' => [],
          'Personnes âgées, malades, immobilisées' => [],
          "Accompagnement, coaching et conseils 'psy'" => [],
          'Santé - Divers' => []
        ],
        'F Administration / Gestion' => [
          '#children' => FALSE,
          'Classement de papiers, de dossiers' => [],
          'Dactylographie, mise en page' => [],
          'Courriers, démarches administratives' => [],
          'Assurances, questions juridiques' => [],
          'Fiscalité, pension, chômage' => [],
          'Administration - Divers' => [],
        ],
        'G Cours / Formations / Conseils' => [
          '#children' => FALSE,
          'Langues, conversation, traduction' => [],
          'Ecriture, rédaction, orthographe' => [],
          "Recherche d'emploi" => [],
          'Gestion et animation de groupes' => [],
          'Cours - Divers' => []
        ],
        'H Jardin / Animaux / Nature' => [
          '#children' => FALSE,
          'Jardin : conception, entretien, conseils' => [],
          'Culture potagère, vergers' => [],
          'Récolte de fruits, légumes, plantes' => [],
          "Taille d'arbres et arbustes" => [],
          'Animaux (domestiques -élevage]' => [],
          'Biodiversité, nichoirs, découverte nature' => [],
          'Engrais, semences, plants, produits divers' => [],
          'Jardin et animaux - Divers' => []
        ],
        'I Arts / Culture / Sports & loisirs' => [
          '#children' => FALSE,
          'Artisanat, peinture, dessin, sculpture' => [],
          'Musique, chant, danse' => [],
          'Photo, vidéo, cinema' => [],
          'Animations, spectacles' => [],
          'Fêtes : conseils, organisation, aide' => [],
          'Promenades, excursions' => [],
          "Jeux (de société, d'extérieur]" => [],
          'Sports' => [],
          'Culture et sport - Divers' => [],
        ],
        'J Transports / Voyages /Hébergement' => [
          '#children' => FALSE,
          'Courses' => [],
          'Covoiturage et transport' => [],
          'Déménagement' => [],
          'Auto-moto-vélo' => [],
          'Voyages, randonnées' => [],
          'Hébergement, échange maisons' => [],
          'Gardiennage maison' => [],
          'Transport - Divers' => []
        ],
        'K Informatique /Électroménager' => [
          '#children' => FALSE,
          'Dépannage informatique' => [],
          'Réparation électroménager' => [],
          'Formations Internet / email' => [],
          'Formations logiciels' => [],
          'Informatique, électroménager - Divers' => [],
        ],
        'Z Divers (non classées]' => [
          '#children' => FALSE,
        ],
      ]
    ];
    return $vocabs;
  }

  function uninstall_devel_generate() {
    $this->installer->uninstall(['devel_generate', 'devel']);
  }

}
